import os

import jinja2


NGINX_SITE_BASE_PATH = '/etc/nginx'
INDENT = ' '*4


class NginxConf:

    def __init__(self, sites_base_path=NGINX_SITE_BASE_PATH):
        self._sites_path = os.path.join(sites_base_path, 'sites-available')

    # Expose sites_path as a property to allow mocking in indirect calls to
    # this class.
    @property
    def sites_path(self):
        return self._sites_path

    def write_site(self, site, new):
        fname = os.path.join(self.sites_path, '{}.conf'.format(site))
        # Check if contents changed
        try:
            with open(fname, 'r', encoding='utf-8') as f:
                current = f.read()
        except FileNotFoundError:
            current = ''
        if new == current:
            return False
        with open(fname, 'w', encoding='utf-8') as f:
            f.write(new)
        return True

    def sync_sites(self, sites):
        changed = False
        for fname in os.listdir(self.sites_path):
            site = fname.replace('.conf', '')
            available = os.path.join(self.sites_path, '{}.conf'.format(site))
            enabled = os.path.join(os.path.dirname(self.sites_path), 'sites-enabled', '{}.conf'.format(site))
            if site not in sites:
                changed = True
                try:
                    os.remove(available)
                    os.remove(enabled)
                except FileNotFoundError:
                    pass
            elif not os.path.exists(enabled):
                changed = True
                os.symlink(available, enabled)

        return changed

    def _generate_name(self, name):
        return name.split('.')[0]

    def render(self, site, listen_port, backend):
        base = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
        env = jinja2.Environment(loader=jinja2.FileSystemLoader(base))
        template = env.get_template('templates/nginx_cfg.tmpl')
        return template.render({
            'site': site,
            'name': self._generate_name(site),
            'port': listen_port,
            'backend': backend,
        })
